package dados.repositorioBD;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import dados.base.Ambiente;
import dados.base.Assento;
import exceptions.AmbienteJahCadastradoException;
import exceptions.AmbienteNaoCadastradoException;
import exceptions.BancoDeDadosException;
import exceptions.EventoNaoCadastradoException;

public class RepositorioAmbienteBD {
	private Connection conexaoBD;
	private PreparedStatement preparedStatement;
	private PreparedStatement preparedStatement2;
	private Statement statement;
	private ResultSet resposta;
	
	public RepositorioAmbienteBD(Connection conexaoBD) {
		this.conexaoBD = conexaoBD;
	}

	public void removerAmbiente(int idAmbiente) throws AmbienteNaoCadastradoException, BancoDeDadosException {
		String removerAssentos = "DELETE FROM Assento WHERE idAmbiente = ?";
		String removerAmbiente = "DELETE FROM Ambiente WHERE idAmbiente = ?";
		String selectEventos = "SELECT idEvento AS evento_id FROM Evento WHERE idAmbiente = ?";
		try {
			try {
				Ambiente ambienteArmazenado = this.consultarAmbiente(idAmbiente);
				this.preparedStatement = this.conexaoBD.prepareStatement(selectEventos);
				this.preparedStatement.setInt(1, idAmbiente);
				this.resposta = this.preparedStatement.executeQuery();	
				if(this.resposta.next() == false){
								
					this.preparedStatement = this.conexaoBD.prepareStatement(removerAssentos);
					this.preparedStatement.setInt(1, idAmbiente);
					this.preparedStatement.execute();
					
					this.preparedStatement = this.conexaoBD.prepareStatement(removerAmbiente);
					this.preparedStatement.setInt(1, idAmbiente);
					this.preparedStatement.execute();
				}else{
					throw new BancoDeDadosException("Existem Eventos cadastrados nesse Ambiente. Remova os eventos antes de remover o ambiente.");
				}
			} catch (AmbienteNaoCadastradoException e1) {
				throw new AmbienteNaoCadastradoException("No existe nenhum Ambiente Cadastrado com Id = " + idAmbiente);
			}
			this.preparedStatement.close();
		} catch (SQLException e) {
			throw new BancoDeDadosException("Banco de Dados Indisponvel.");
		}
		
		
		/*finally{
			try {
			} catch (SQLException e) {
				throw new BancoDeDadosException("Banco de Dados Indisponvel.");
			}
		}*/
	}

	public void inserirAmbiente(Ambiente ambiente) throws AmbienteJahCadastradoException, BancoDeDadosException {
		int idAmbiente = ambiente.getIdAmbiente();
		int linhas = ambiente.getLinhas();
		int colunas = ambiente.getColunas();
		
		String insertAmbiente = "INSERT INTO Ambiente (idAmbiente, filas, colunas) VALUES (" + ambiente.getIdAmbiente() + "," +  ambiente.getLinhas() + "," + ambiente.getColunas() +")";
		String insertAssentos = "";
		try {
			this.statement = this.conexaoBD.createStatement();
			this.statement.executeUpdate(insertAmbiente);
		} catch (SQLException e) {
			throw new AmbienteJahCadastradoException("O ambiente com Id = " + idAmbiente + " ja foi cadastrado anteriormente.");
		}	
		
		try {
			for(int i = 0; i < linhas; i++){
				for(int j = 0; j < colunas; j++){
					insertAssentos = "INSERT INTO Assento(linha, coluna,  idAmbiente) VALUES (" + (i+1) + "," + (j+1) + "," + idAmbiente + ")";
					this.statement.executeUpdate(insertAssentos);			
				}	
			}
			
			this.statement.close();
		} catch (SQLException e) {
			throw new BancoDeDadosException("Banco de Dados Indisponvel.");
		}
			
				
						
	}


	public Ambiente consultarAmbiente(int idAmbiente) throws AmbienteNaoCadastradoException, BancoDeDadosException {
		Ambiente retorno = null;
		int linhas = 0;
		int colunas = 0;
		String selectAmbiente = "SELECT a.filas AS ambiente_filas, a.colunas AS ambiente_colunas FROM Ambiente a WHERE idAmbiente = ?";
	//	String selectAssento = "SELECT s.idEvento AS assento_evento, s.status AS assento_status FROM Assento a , StatusAssento s  WHERE a.linha = ? AND a.coluna = ? AND s.linha = ? AND s.coluna = ? AND idAmbiente = ? ";
		String selectAssento;
		try {
			this.preparedStatement = this.conexaoBD.prepareStatement(selectAmbiente);
			this.preparedStatement.setInt(1, idAmbiente);
			this.resposta = this.preparedStatement.executeQuery();
			Boolean existe = this.resposta.next();
			if(existe != false){
				linhas = this.resposta.getInt("ambiente_filas");
				colunas = this.resposta.getInt("ambiente_colunas");
				Assento[][] assentos = new Assento[linhas][colunas];
				Assento assento;
				for(int i = 0; i < linhas; i++){
					for(int j = 0; j < colunas; j++){
						selectAssento = "SELECT s.idEvento AS assento_evento, s.status AS assento_status FROM Evento e, StatusAssento s WHERE e.idAmbiente = "+ idAmbiente +" AND s.linha = "+ (i + 1)+" AND s.coluna = "+ (j + 1) +" AND e.idEvento = s.idEvento";
						this.statement = this.conexaoBD.createStatement();
						this.resposta = this.statement.executeQuery(selectAssento);
					//this.preparedStatement = this.conexaoBD.prepareStatement(selectAssento);
					//	this.preparedStatement.setInt(1, idAmbiente);
					//	this.preparedStatement.setInt(2, i + 1);
					//	this.preparedStatement.setInt(3, j + 1);
					//	this.resposta = this.preparedStatement.executeQuery();
						
						
						assento = new Assento(i +1, j +1);
						while(this.resposta.next() != false){
							boolean status =this.resposta.getBoolean("assento_status");
							int idEvento = this.resposta.getInt("assento_evento");
							assento.setStatus(status, idEvento);
							assentos[i][j] = assento;
							
						}
						this.statement.close();
					}
				}	
				
				retorno = new Ambiente(idAmbiente, linhas, colunas);
				retorno.setAssentos(assentos);
			}else{
				throw new AmbienteNaoCadastradoException("No existe nenhum Ambiente Cadastrado com Id = " + idAmbiente + ".");
			}
		} catch (SQLException e) {
			throw new BancoDeDadosException("Banco de Dados Indisponvel.");
		}
		return retorno;
	}

	public void inserirLinha(int idAmbiente) throws AmbienteNaoCadastradoException, BancoDeDadosException {
		Ambiente ambiente = this.consultarAmbiente(idAmbiente);
		int numLinhas = ambiente.getLinhas();
		int linha = numLinhas +1;
		String insertAssentos = "INSERT INTO Assento(linha, coluna,  idAmbiente) VALUES (?, ?, ?)";
		String selectEventos = "SELECT idEvento AS evento_id FROM Evento WHERE idAmbiente = ?";
		String updateAmbiente = "UPDATE Ambiente SET filas = filas + 1 WHERE idAmbiente = " + idAmbiente;
		String insertStatusAssentos = "INSERT INTO StatusAssento(linha, coluna, idEvento, status) VALUES (?, ?, ?, ?)";
		int idEvento;
		try {
			
			this.statement = this.conexaoBD.createStatement();
			this.statement.executeUpdate(updateAmbiente);			
			this.statement.close();
			
			for(int j = 0; j < ambiente.getColunas(); j++){
				this.preparedStatement = this.conexaoBD.prepareStatement(insertAssentos);
				this.preparedStatement.setInt(1, linha);
				this.preparedStatement.setInt(2, j + 1);
				this.preparedStatement.setInt(3, idAmbiente);
				this.preparedStatement.execute();			
			}
			
			this.preparedStatement = this.conexaoBD.prepareStatement(selectEventos);
			this.preparedStatement.setInt(1, idAmbiente);
			this.resposta = this.preparedStatement.executeQuery();
			while(this.resposta.next() != false){
				idEvento = this.resposta.getInt("evento_id");
				for(int j = 0; j < ambiente.getColunas(); j++){
					this.preparedStatement = this.conexaoBD.prepareStatement(insertStatusAssentos);
					this.preparedStatement.setInt(1, linha);
					this.preparedStatement.setInt(2, j + 1);
					this.preparedStatement.setInt(3, idEvento);
					this.preparedStatement.setInt(4, 0);
					this.preparedStatement.execute();	
					this.preparedStatement.close();
				}				
			}	
			
		} catch (SQLException e) {
			throw new BancoDeDadosException("Banco de Dados Indisponvel.");
		}
	}

	
	
	
	
	
	public void inserirColuna(int idAmbiente) throws AmbienteNaoCadastradoException, BancoDeDadosException{
		Ambiente ambiente = this.consultarAmbiente(idAmbiente);
		int numColunas = ambiente.getColunas();
		int coluna = numColunas +1;
		String insertAssentos = "INSERT INTO Assento(linha, coluna,  idAmbiente) VALUES (?, ?, ?)";
		String selectEventos = "SELECT idEvento AS evento_id FROM Evento WHERE idAmbiente = ?";
		String updateAmbiente = "UPDATE Ambiente SET colunas = colunas + 1 WHERE idAmbiente = " + idAmbiente;
		String insertStatusAssentos = "INSERT INTO StatusAssento(linha, coluna, idEvento, status) VALUES (?, ?, ?, ?)";
		int idEvento;
		try {
			
			this.statement = this.conexaoBD.createStatement();
			this.statement.executeUpdate(updateAmbiente);			
			this.statement.close();
			
			for(int i = 0; i < ambiente.getLinhas(); i++){
				this.preparedStatement = this.conexaoBD.prepareStatement(insertAssentos);
				this.preparedStatement.setInt(1, i + 1);
				this.preparedStatement.setInt(2, coluna);
				this.preparedStatement.setInt(3, idAmbiente);
				this.preparedStatement.execute();		
				this.preparedStatement.close();
			}
			
			this.preparedStatement = this.conexaoBD.prepareStatement(selectEventos);
			this.preparedStatement.setInt(1, idAmbiente);
			this.resposta = this.preparedStatement.executeQuery();
			int count = 0;
			while(this.resposta.next() != false){
				count++;
				idEvento = this.resposta.getInt("evento_id");
				for(int i = 0; i < ambiente.getLinhas(); i++){
					this.preparedStatement = this.conexaoBD.prepareStatement(insertStatusAssentos);
					this.preparedStatement.setInt(1, i+1);
					this.preparedStatement.setInt(2, coluna);
					this.preparedStatement.setInt(3, idEvento);
					this.preparedStatement.setInt(4, 0);
					this.preparedStatement.execute();	
				}				
			}	
			
			this.preparedStatement.close();
		} catch (SQLException e) {
			throw new BancoDeDadosException("Banco de Dados Indisponvel.");
		}
		
	}
	
	public void removerLinha(int idAmbiente) throws AmbienteNaoCadastradoException, BancoDeDadosException {
		Ambiente ambiente = this.consultarAmbiente(idAmbiente);
		int linha = ambiente.getLinhas();
		String updateAmbiente = "UPDATE Ambiente SET filas = filas - 1 WHERE idAmbiente = " + idAmbiente;
		String deleteAssentos = "DELETE FROM ASSENTO WHERE idAmbiente = " + idAmbiente + " AND linha = "+ linha;
		String deleteStatusAssento = "DELETE FROM StatusAssento WHERE idEvento = ? AND linha = ?";
		String selectEventos = "SELECT idEvento AS evento_id FROM Evento WHERE idAmbiente = ?";
		int idEvento;
		try {
			
			this.statement = this.conexaoBD.createStatement();
			this.statement.executeUpdate(updateAmbiente);			
			this.statement.close();		
			
			this.statement = this.conexaoBD.createStatement();
			this.statement.executeUpdate(deleteAssentos);			
			this.statement.close();	
			
			
			this.preparedStatement = this.conexaoBD.prepareStatement(selectEventos);
			this.preparedStatement.setInt(1, idAmbiente);
			this.resposta = this.preparedStatement.executeQuery();
			
			while(this.resposta.next() != false){
				idEvento = this.resposta.getInt("evento_id");
				this.preparedStatement = this.conexaoBD.prepareStatement(deleteStatusAssento);
				this.preparedStatement.setInt(1, idEvento);
				this.preparedStatement.setInt(2, linha);
				this.preparedStatement.execute();
				this.preparedStatement.close();
			}			
			
		} catch (SQLException e) {
			throw new BancoDeDadosException("Banco de Dados Indisponvel.");
		}
	}
	
	public void removerColuna(int idAmbiente) throws AmbienteNaoCadastradoException, BancoDeDadosException {
		Ambiente ambiente = this.consultarAmbiente(idAmbiente);
		int coluna = ambiente.getColunas();
		String updateAmbiente = "UPDATE Ambiente SET colunas = colunas - 1 WHERE idAmbiente = " + idAmbiente;
		String deleteAssentos = "DELETE FROM ASSENTO WHERE idAmbiente = "+ idAmbiente +" AND coluna = "+ coluna;
		String deleteStatusAssento = "DELETE FROM StatusAssento WHERE idEvento = ? AND coluna = ?";
		String selectEventos = "SELECT idEvento AS evento_id FROM Evento WHERE idAmbiente = ?";
		int idEvento;
		try {
			
			this.statement = this.conexaoBD.createStatement();
			this.statement.executeUpdate(updateAmbiente);			
			this.statement.close();		
			
			this.statement = this.conexaoBD.createStatement();
			this.statement.executeUpdate(deleteAssentos);			
			this.statement.close();	
			
			this.preparedStatement = this.conexaoBD.prepareStatement(selectEventos);
			this.preparedStatement.setInt(1, idAmbiente);
			this.resposta = this.preparedStatement.executeQuery();
			
			while(this.resposta.next() != false){
				idEvento = this.resposta.getInt("evento_id");
				this.preparedStatement = this.conexaoBD.prepareStatement(deleteStatusAssento);
				this.preparedStatement.setInt(1, idEvento);
				this.preparedStatement.setInt(2, coluna);
				this.preparedStatement.execute();
				this.preparedStatement.close();
			}			
			
		} catch (SQLException e) {
			throw new BancoDeDadosException("Banco de Dados Indisponvel.");
		}
	}


	/*public Vector<Ambiente> listarAmbientes() throws BancoDeDadosException, AmbienteNaoCadastradoException {
		Vector<Ambiente> retorno = new Vector<Ambiente>();
		String selectIdsAmbiente = "SELECT idAmbiente FROM AMBIENTE";

		try {
			this.preparedStatement2 = this.conexaoBD.prepareStatement(selectIdsAmbiente);
			this.resposta = this.preparedStatement2.executeQuery();
			try {
				while(this.resposta.next() != false){
					System.out.println("looop");
					int idAmbiente = this.resposta.getInt("idAmbiente");
					System.out.println("idAmbiente = " + idAmbiente);
					retorno.add(this.consultarAmbiente(idAmbiente));
				}
				this.preparedStatement2.close();			
			} catch (AmbienteNaoCadastradoException e) {
				throw new AmbienteNaoCadastradoException("No existe nenhum Ambiente cadastrado");
			} 
		} catch (SQLException e) {
			e.printStackTrace();
			throw new BancoDeDadosException("Banco de dados indisponvel.");
		}
		
		return retorno;
	}*/
	
	public Vector<Ambiente> listarAmbientes() throws BancoDeDadosException{
		Vector<Ambiente> retorno = new Vector<Ambiente>();
		Vector<Integer> listaDeIds = this.listarIDs();
		for(int i = 0; i < listaDeIds.size(); i++){
			try {
				retorno.add(this.consultarAmbiente(listaDeIds.get(i)));
			} catch (AmbienteNaoCadastradoException e) {
				//nao faz nada se n tiver nenhum ambiente cadastrado
			} catch (BancoDeDadosException e) {
				new BancoDeDadosException("Banco de dados indisponvel.");
			}
		}
		
		return retorno;
	}
	
	private Vector<Integer> listarIDs() throws BancoDeDadosException{
		Vector<Integer> retorno = new Vector<Integer>();
		String selectIdsAmbiente = "SELECT idAmbiente FROM AMBIENTE";	
	
		try {
			this.preparedStatement = this.conexaoBD.prepareStatement(selectIdsAmbiente);
		this.resposta = this.preparedStatement.executeQuery();
		
			while(this.resposta.next() != false){
				int idAmbiente = this.resposta.getInt("idAmbiente");
				retorno.add(idAmbiente);
			}
		} catch (SQLException e) {
			throw new BancoDeDadosException("Banco de dados indisponvel");
		}
		return retorno;
	}
}
