package dados.repositorioBD;


import dados.base.Data;
import dados.base.Evento;
import dados.base.Hora;
import exceptions.AmbienteNaoCadastradoException;
import exceptions.BancoDeDadosException;
import exceptions.EventoJahCadastradoException;
import exceptions.EventoNaoCadastradoException;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import negocios.ControleAmbiente;

public class RepositorioEventoBD {
	private Connection conexaoBD;
	private Statement statement;
	private PreparedStatement preparedStatement;
	private ResultSet resposta;
	ControleAmbiente controleAmbiente;
	
	public RepositorioEventoBD(Connection conexao, ControleAmbiente controleAmbiente){
		this.conexaoBD = conexao;
		this.controleAmbiente = controleAmbiente;
	}
	
	public void inserirEvento(Evento evento) throws EventoJahCadastradoException, AmbienteNaoCadastradoException, BancoDeDadosException{
		String insert = "INSERT INTO Evento (idEvento, dataEvento, hora, nome, preco, idAmbiente) VALUES ("+ evento.getId() + ", to_date('" + evento.getData().getDia() + "/" + evento.getData().getMes() + "/" + evento.getData().getAno() + " ' , 'DD/MM/YYYY' ), to_date('" + evento.getHora().getHoras() + ":" +evento.getHora().getMinutos()+"', 'HH24:MI'), '" + evento.getNome() +"', "+ evento.getPreco() + ", " +evento.getAmbiente().getIdAmbiente() +")";
		int idAmbiente = evento.getAmbiente().getIdAmbiente();
		try {
			this.controleAmbiente.consultarAmbiente(idAmbiente);
			
			try {
				this.statement = this.conexaoBD.createStatement();
				this.statement.executeUpdate(insert);
				
				int linhas = evento.getAmbiente().getLinhas();
				int colunas =  evento.getAmbiente().getColunas();
				
				for(int i = 1; i <= linhas; i++){
					for(int j = 1; j <= colunas; j++){
						this.statement.executeUpdate("INSERT INTO StatusAssento(linha, coluna, idEvento, status) VALUES (" + i + "," + j + "," + evento.getId() +"," +"0)");
					}
				}			
				this.statement.close();
			} catch (SQLException e) {
				if(e.getMessage().compareTo("ORA-00001: unique constraint (G092IF682_EQ08.DATAEHORAUNICAS) violated\n") == 0){
					throw new EventoJahCadastradoException(" Um evento na sala " + evento.getAmbiente().getIdAmbiente() +" com Data: " + Data .dataToString(evento.getData()) + " e Hora: " + Hora.horaToStringStatic(evento.getHora()) + " ja foi cadastrado anteriormente.");
				}else if(e.getMessage().compareTo("ORA-00001: unique constraint (G092IF682_EQ08.CHAVEPRIMARIAEVENTO) violated\n") == 0){	
					throw new EventoJahCadastradoException(" O evento com Id: " + evento.getId() +" ja foi cadastrado anteriormente.");
				}else{
					throw new EventoJahCadastradoException(" O evento com Id: " + evento.getId() +" ou Data: " + Data .dataToString(evento.getData()) + " e Hora: " + Hora.horaToStringStatic(evento.getHora()) + " ja foi cadastrado anteriormente.");

				}
			}
		} catch (AmbienteNaoCadastradoException e1) {
			throw new AmbienteNaoCadastradoException("No existe nenhum Ambiente Cadastrado com Id = " + idAmbiente);
		} catch (BancoDeDadosException e1) {
			throw new BancoDeDadosException("Banco de Dados Indisponvel.");
		}
		
		
	}	
	
	public void removerEvento(int id) throws EventoNaoCadastradoException, BancoDeDadosException{
		String remover1 = "DELETE FROM StatusAssento WHERE idEvento = " + id;
		String remover2 = "DELETE FROM Evento WHERE idEvento = " + id;
		
		try {
			this.consultarEvento(id);
			try {
				this.statement = this.conexaoBD.createStatement();
				this.statement.executeUpdate(remover1);
				this.statement.executeUpdate(remover2);
			} catch (SQLException e) {
				throw new BancoDeDadosException("Banco de Dados Indisponvel.");
			}
			
			finally{
				try {
					this.statement.close();
				} catch (SQLException e) {
					throw new BancoDeDadosException("Banco de Dados Indisponvel.");
				}
			}
		} catch (EventoNaoCadastradoException e1) {
			throw new EventoNaoCadastradoException("No existe Evento cadastrado com Id = " + id + ".");
		} catch (AmbienteNaoCadastradoException e1) {
			throw new EventoNaoCadastradoException("No existe Evento cadastrado com Id = " + id + ".");
		}
	}
	
	public Evento consultarEvento(int idEvento) throws EventoNaoCadastradoException, AmbienteNaoCadastradoException, BancoDeDadosException{
		Evento retorno = null;
		String select = "SELECT idAmbiente, dataEvento, hora, nome, preco FROM Evento WHERE idEvento = ?";
		String data;
		String hora;
		Data data2 = null;
		Hora hora2 = null;
		String nome;
		int preco;
		int idAmbiente;
		try {
			this.preparedStatement = this.conexaoBD.prepareStatement(select);
			this.preparedStatement.setInt(1, idEvento);
			this.resposta = this.preparedStatement.executeQuery();
			Boolean existe = this.resposta.next();		
			if(existe != false){
				idAmbiente = this.resposta.getInt("idAmbiente");
				data = this.resposta.getDate("dataEvento").toString();
				hora = this.resposta.getTime("hora").toString();
				nome = this.resposta.getString("nome");
				preco = this.resposta.getInt("preco");
				data2 = Data.stringToData(data);
				hora2 = Hora.stringToHora(hora);

				retorno = new Evento(idEvento, this.controleAmbiente.consultarAmbiente(idAmbiente), data2, hora2, nome, preco);
			}else{
				throw new EventoNaoCadastradoException("No existe Evento cadastrado com Id = " + idEvento + ".");
			}
			this.preparedStatement.close();
		} catch (SQLException e) {
			throw new BancoDeDadosException("Banco de Dados Indisponvel.");
		}
		
		return retorno;
	}
	
	public Vector<Evento> consultarEventosPeloNome(String nomeEvento) throws BancoDeDadosException{
		Vector<Evento> retorno = new Vector<Evento>();
		Vector<Integer> listaDeIds = this.listarIDsComNome(nomeEvento);
			for(int i = 0; i < listaDeIds.size(); i++){
				try {
					retorno.add(this.consultarEvento((listaDeIds.get(i))));
				} catch (EventoNaoCadastradoException e) {
					//nao faz nada se n tiver evento cadastrado com esse nome.
				} catch (AmbienteNaoCadastradoException e) {
				} catch (BancoDeDadosException e) {
					throw new BancoDeDadosException("Banco de dados indisponvel");
				}
			}	
		return retorno;
	}
	
	private Vector<Integer> listarIDsComNome(String nomeEvento) throws BancoDeDadosException{
		Vector<Integer> retorno = new Vector<Integer>();
		String selectIdsEvento = "SELECT idEvento FROM Evento WHERE nome LIKE '%"+nomeEvento+"%'";
	
	
		try {
			this.statement = this.conexaoBD.createStatement();
			this.resposta = this.statement.executeQuery(selectIdsEvento);
			
			//this.preparedStatement = this.conexaoBD.prepareStatement(selectIdsEvento);
			//this.preparedStatement.setString(1, nomeEvento);
			//this.resposta = this.preparedStatement.executeQuery();
			
			if(this.resposta.next() != false){
				int idEvento = this.resposta.getInt("idEvento");
				retorno.add(idEvento);
			}
			this.statement.close();
		} catch (SQLException e) {
			throw new BancoDeDadosException("Banco de dados indisponvel");
		}
		return retorno;
	}
	
	public void inserirReserva(int idEvento, int linha, int coluna) throws BancoDeDadosException{
		String update = "UPDATE StatusAssento SET  status = 1 WHERE (linha = ? AND coluna = ? AND idEvento = ?)";
		try {
			this.preparedStatement = this.conexaoBD.prepareStatement(update);
			int parameterIndex = 1;
			this.preparedStatement.setInt(parameterIndex++, linha);
			this.preparedStatement.setInt(parameterIndex++, coluna);
			this.preparedStatement.setInt(parameterIndex++, idEvento);
			this.preparedStatement.execute();
		} catch (SQLException e) {
			throw new BancoDeDadosException("Banco de Dados Indisponvel.");
		}
		
		finally{
			try {
				this.preparedStatement.close();
			} catch (SQLException e) {
				throw new BancoDeDadosException("Banco de Dados Indisponvel.");
			}
		}
	}
	
/*	public Vector<Evento> listarEventos() throws EventoNaoCadastradoException, AmbienteNaoCadastradoException, BancoDeDadosException{
		Vector<Evento> retorno = new Vector<Evento>();
		String selectIdsEventos = "SELECT idEvento FROM Evento";
		
		try {
			this.preparedStatement = this.conexao.prepareStatement(selectIdsEventos);
			this.respota = this.preparedStatement.executeQuery();
			try {
				while(this.respota.next()){
					int idEvento = this.respota.getInt("idEvento");
					retorno.add(this.consultarEvento(idEvento));
				}
				this.preparedStatement.close();
			} catch (EventoNaoCadastradoException e) {
				e.printStackTrace();
				throw new EventoNaoCadastradoException("No existe nenhum Evento cadastrado");
			} 
		} catch (SQLException e) {
			throw new BancoDeDadosException("Banco de dados indisponvel.");
		}
		
		return retorno;
	}*/
	
	
	public Vector<Evento> listarEventos() throws BancoDeDadosException{
		Vector<Evento> retorno = new Vector<Evento>();
		Vector<Integer> listaDeIds = this.listarIDs();
		try {
			for(int i = 0; i < listaDeIds.size(); i++){
				try {
					retorno.add(this.consultarEvento((listaDeIds.get(i))));
				} catch (AmbienteNaoCadastradoException e) {
				}
			}
		} catch (EventoNaoCadastradoException e) {
			//nao faz nada se n encontrar nenhum Evento
		} catch (BancoDeDadosException e) {
			throw new BancoDeDadosException("Banco de dados indisponvel");
		}
		
		return retorno;
	}
	
	private Vector<Integer> listarIDs() throws BancoDeDadosException{
		Vector<Integer> retorno = new Vector<Integer>();
		String selectIdsEvento = "SELECT idEvento FROM Evento";
	
	
		try {
			this.preparedStatement = this.conexaoBD.prepareStatement(selectIdsEvento);
			this.resposta = this.preparedStatement.executeQuery();
			
			while(this.resposta.next() != false){
				int idEvento = this.resposta.getInt("idEvento");
				retorno.add(idEvento);
			}
		} catch (SQLException e) {
			throw new BancoDeDadosException("Banco de dados Indisponvel.");
		}
		return retorno;
	}
	

}
