
package fachada;

import java.sql.Connection;
import java.util.Vector;

import dados.base.*;
import exceptions.AmbienteJahCadastradoException;
import exceptions.AmbienteNaoCadastradoException;
import exceptions.BancoDeDadosException;
import exceptions.ControleExcepttion;
import exceptions.EventoJahCadastradoException;
import exceptions.EventoNaoCadastradoException;
import exceptions.FuncionarioJahCadastradoException;
import exceptions.FuncionarioNaoCadastradoException;
import negocios.*;

public class Fachada {
	private ControleEvento controleEvento;
	private ControleAmbiente controleAmbiente;
	private ControleFuncionario controleFuncionario;
	private Connection conexaoBD;
	
	public Fachada(Connection conexaoBD) {
		this.conexaoBD = conexaoBD;
		this.controleAmbiente = new ControleAmbiente(this.conexaoBD);
		this.controleEvento = new ControleEvento(this.conexaoBD, this.controleAmbiente);
		this.controleFuncionario = new ControleFuncionario(this.conexaoBD);
	}
	
	//Mtodos de Ambiente
	
	public void inserirAmbiente(int idAmbiente, int linha, int coluna) throws AmbienteJahCadastradoException, BancoDeDadosException, ControleExcepttion {
		Ambiente ambiente = new Ambiente(idAmbiente, linha, coluna);
		this.controleAmbiente.inserirAmbiente(ambiente);
	}
	
	public void atualizarAmbiente(Ambiente antigoAmbiente, Ambiente novoAmbiente) throws AmbienteJahCadastradoException, BancoDeDadosException, ControleExcepttion, AmbienteNaoCadastradoException {
		this.removerAmbiente(antigoAmbiente.getIdAmbiente());
		
		try {
			this.inserirAmbiente(novoAmbiente.getIdAmbiente(), novoAmbiente.getLinhas(), novoAmbiente.getColunas());
		} catch (AmbienteJahCadastradoException e) {
			this.inserirAmbiente(antigoAmbiente.getIdAmbiente(), antigoAmbiente.getLinhas(), antigoAmbiente.getColunas());
			throw new AmbienteJahCadastradoException(e.getMessage());
		} catch (BancoDeDadosException e) {
			this.inserirAmbiente(antigoAmbiente.getIdAmbiente(), antigoAmbiente.getLinhas(), antigoAmbiente.getColunas());
			throw new BancoDeDadosException(e.getMessage());
		} catch (ControleExcepttion e) {
			this.inserirAmbiente(antigoAmbiente.getIdAmbiente(), antigoAmbiente.getLinhas(), antigoAmbiente.getColunas());
			throw new ControleExcepttion(e.getMessage());
		}
	}
	
	public void removerAmbiente(int idAmbiente) throws AmbienteNaoCadastradoException, BancoDeDadosException {
		this.controleAmbiente.removerAmbiente(idAmbiente);
	}
	
	public Ambiente consultarAmbiente(int idAmbiente) throws AmbienteNaoCadastradoException, BancoDeDadosException {
		return this.controleAmbiente.consultarAmbiente(idAmbiente);
	}
	
	//Em rela aos assentos de um determinado ambiente
	
	public void inserirLinhas(int idAmbiente, int linhas) throws AmbienteNaoCadastradoException, BancoDeDadosException, ControleExcepttion {
		this.controleAmbiente.inserirLinhas(idAmbiente, linhas);
	}
	
	public void removerLinhas(int idAmbiente, int linhas) throws AmbienteNaoCadastradoException, BancoDeDadosException, ControleExcepttion {
		this.controleAmbiente.removerLinhas(idAmbiente, linhas);
	}
	
	public void inserirColunas(int idAmbiente, int colunas) throws AmbienteNaoCadastradoException, BancoDeDadosException, ControleExcepttion {
		this.controleAmbiente.inserirColunas(idAmbiente, colunas);
	}
	
	public void removerColunas(int idAmbiente, int colunas) throws AmbienteNaoCadastradoException, BancoDeDadosException, ControleExcepttion {
		this.controleAmbiente.removerColunas(idAmbiente, colunas);
	}
	
	
	public void inserirLinhasEColunas(int idAmbiente, int linhas, int colunas) throws AmbienteNaoCadastradoException, BancoDeDadosException, ControleExcepttion{
		
		this.inserirLinhas(idAmbiente, linhas);
		//tenta inserir colunas
		try {
			this.inserirColunas(idAmbiente, colunas);
			
			//se nao conseguir remove as linhas que foram inseridas
		} catch (AmbienteNaoCadastradoException e) {
			this.removerLinhas(idAmbiente, linhas);
			throw new AmbienteNaoCadastradoException(e.getMessage());
		} catch (BancoDeDadosException e) {
			this.removerLinhas(idAmbiente, linhas);
			throw new BancoDeDadosException(e.getMessage());
		} catch (ControleExcepttion e) {
			this.removerLinhas(idAmbiente, linhas);
			throw new ControleExcepttion(e.getMessage());
		}
	}
	
	public void removerLinhasEColunas(int idAmbiente, int linhas, int colunas) throws AmbienteNaoCadastradoException, BancoDeDadosException, ControleExcepttion{
		this.removerLinhas(idAmbiente, linhas);
		
		try {
			this.removerColunas(idAmbiente, colunas);
		} catch (AmbienteNaoCadastradoException e) {
			this.inserirLinhas(idAmbiente, linhas);
			throw new AmbienteNaoCadastradoException(e.getMessage());
		} catch (BancoDeDadosException e) {
			this.inserirLinhas(idAmbiente, linhas);
			throw new BancoDeDadosException(e.getMessage());
		} catch (ControleExcepttion e) {
			this.inserirLinhas(idAmbiente, linhas);
			throw new ControleExcepttion(e.getMessage());
		}
	}
	
	//Mtodos Funcionrio
	
	public void inserirFuncionario(String nome, String email, String senha, String CPF, Endereco endereco, Telefone telefone) throws FuncionarioJahCadastradoException, ControleExcepttion {
		Funcionario funcionario = new Funcionario(nome, email, senha, CPF, endereco, telefone);
		this.controleFuncionario.inserirFuncionario(funcionario);
	}
	
	public void removerFuncionario(String CPF) throws FuncionarioNaoCadastradoException, BancoDeDadosException, ControleExcepttion {
		this.controleFuncionario.removerFuncionario(CPF);
	}
	
	public Funcionario consultarFuncionario(String CPF) throws FuncionarioNaoCadastradoException, BancoDeDadosException {
		return this.controleFuncionario.consultarFuncionario(CPF);
	}
	
	public Vector<Funcionario> consultarFuncionarioPeloNome(String nomeDoFuncionario) throws BancoDeDadosException{
		return this.controleFuncionario.consultarFuncionariosPeloNome(nomeDoFuncionario);
	}
	
	public void atualizarFuncionario(Funcionario antigoFuncionario, Funcionario novoFuncionario) throws FuncionarioJahCadastradoException, ControleExcepttion, FuncionarioNaoCadastradoException, BancoDeDadosException{
		this.removerFuncionario(antigoFuncionario.getCpf());
	
		try {
			this.inserirFuncionario(novoFuncionario.getNome(), novoFuncionario.getEmail(), novoFuncionario.getSenha(), novoFuncionario.getCpf(), novoFuncionario.getEndereco(), novoFuncionario.getTelefone());
		} catch (FuncionarioJahCadastradoException e) {
			this.inserirFuncionario(antigoFuncionario.getNome(), antigoFuncionario.getEmail(), antigoFuncionario.getSenha(), antigoFuncionario.getCpf(), antigoFuncionario.getEndereco(), antigoFuncionario.getTelefone());
			throw new FuncionarioJahCadastradoException(e.getMessage());
		} catch (ControleExcepttion e) {
			this.inserirFuncionario(antigoFuncionario.getNome(), antigoFuncionario.getEmail(), antigoFuncionario.getSenha(), antigoFuncionario.getCpf(), antigoFuncionario.getEndereco(), antigoFuncionario.getTelefone());
			throw new ControleExcepttion(e.getMessage());
		}
	}
	
	//Metodos Evento
	
	public void inserirEvento(Data data, Hora hora, int idAmbiente, int idEvento, String nome, int preco) throws EventoJahCadastradoException, AmbienteNaoCadastradoException, BancoDeDadosException, ControleExcepttion {
		Evento evento = new Evento(idEvento,this.consultarAmbiente(idAmbiente) ,data,hora,nome, preco);
		this.controleEvento.inserirEvento(evento);
	}
	
	public void removerEvento(int idEvento) throws EventoNaoCadastradoException, BancoDeDadosException {
		this.controleEvento.removerEvento(idEvento);
	}
	
	public void atualizarEvento(Evento antigoEvento, Evento novoEvento) throws EventoNaoCadastradoException, BancoDeDadosException, EventoJahCadastradoException, AmbienteNaoCadastradoException, ControleExcepttion {

			this.removerEvento(antigoEvento.getId());			
			try {
				this.inserirEvento(novoEvento.getData(), novoEvento.getHora(), novoEvento.getAmbiente().getIdAmbiente(), novoEvento.getId(), novoEvento.getNome(), novoEvento.getPreco());
		
			} catch (EventoJahCadastradoException e) {
				this.inserirEvento(antigoEvento.getData(), antigoEvento.getHora(), antigoEvento.getAmbiente().getIdAmbiente(), antigoEvento.getId(), antigoEvento.getNome(), antigoEvento.getPreco());		
				throw new EventoJahCadastradoException(e.getMessage());
			} catch (AmbienteNaoCadastradoException e) {
				this.inserirEvento(antigoEvento.getData(), antigoEvento.getHora(), antigoEvento.getAmbiente().getIdAmbiente(), antigoEvento.getId(), antigoEvento.getNome(), antigoEvento.getPreco());
				throw new AmbienteNaoCadastradoException(e.getMessage());
			} catch (BancoDeDadosException e) {
				this.inserirEvento(antigoEvento.getData(), antigoEvento.getHora(), antigoEvento.getAmbiente().getIdAmbiente(), antigoEvento.getId(), antigoEvento.getNome(), antigoEvento.getPreco());
				throw new BancoDeDadosException(e.getMessage());
			} catch (ControleExcepttion e) {
				this.inserirEvento(antigoEvento.getData(), antigoEvento.getHora(), antigoEvento.getAmbiente().getIdAmbiente(), antigoEvento.getId(), antigoEvento.getNome(), antigoEvento.getPreco());
				throw new ControleExcepttion(e.getMessage());
			}
	}
	
	
	public Evento consultarEvento(int idEvento) throws EventoNaoCadastradoException, AmbienteNaoCadastradoException, BancoDeDadosException{
		return this.controleEvento.consultarEvento(idEvento);
	}
	
	public Vector<Evento> consultarEventosPeloNome(String nomeEvento) throws BancoDeDadosException{
		return this.controleEvento.consultarEventosPeloNome(nomeEvento);
	}
	
	public boolean verificarSenha(String CPF, String senha) throws FuncionarioNaoCadastradoException, BancoDeDadosException {
		return this.controleFuncionario.verificarSenha(CPF,senha);
		
	}
	
	public Vector<Ambiente> listarAmbientes() throws BancoDeDadosException{
		return this.controleAmbiente.listarAmbientes();
	}
	
	public Vector<Evento> listarEventos() throws BancoDeDadosException{
		return this.controleEvento.listarEventos();
	}
		

	public void inserirReserva(int idEvento, int linha, int coluna) throws BancoDeDadosException{
		this.controleEvento.inserirReserva(idEvento, linha, coluna);
	}
}
