package negocios;


import java.sql.Connection;
import java.util.Vector;

import dados.base.Ambiente;
import dados.repositorio.RepositorioAmbiente;
import exceptions.AmbienteJahCadastradoException;
import exceptions.AmbienteNaoCadastradoException;
import exceptions.BancoDeDadosException;
import exceptions.ControleExcepttion;

public class ControleAmbiente {
	private RepositorioAmbiente repositorioAmbiente;
	private static final int numMaxLinhas = 10;
	private static final int numMaxColunas = 15;
	
	public ControleAmbiente(Connection conexaoBD) {
		this.repositorioAmbiente = new RepositorioAmbiente(conexaoBD);
	}
	
	public void inserirAmbiente(Ambiente ambiente) throws AmbienteJahCadastradoException, BancoDeDadosException, ControleExcepttion {
		if(ambiente.getLinhas() > 0 && ambiente.getColunas() > 0){
			if(ambiente.getLinhas() <= numMaxLinhas && ambiente.getColunas() <= numMaxColunas){
				this.repositorioAmbiente.inserirAmbiente(ambiente);
			}else if(ambiente.getLinhas() <= numMaxLinhas && ambiente.getColunas() > numMaxColunas){
				throw new ControleExcepttion("O nmero de assentos superou a capacidade mxima! Diminua o nmero de colunas.");
			}else if(ambiente.getLinhas() > numMaxLinhas && ambiente.getColunas() <= numMaxColunas){
				throw new ControleExcepttion("O nmero de assentos superou a capacidade mxima! Diminua o nmero de linhas.");
			}else{
				throw new ControleExcepttion("O nmero de assentos superou a capacidade mxima! Diminua o nmero de linhas e colunas.");
			}
		}else{
			throw new ControleExcepttion("O nmero de linhas e colunas de assentos no ambiente deve ser maior que zero!");
		}
	}
	
	
	public void removerAmbiente(int idAmbiente) throws AmbienteNaoCadastradoException, BancoDeDadosException {
		this.repositorioAmbiente.removerAmbiente(idAmbiente);
	}
	
	public Ambiente consultarAmbiente(int idAmbiente) throws AmbienteNaoCadastradoException, BancoDeDadosException {
		return this.repositorioAmbiente.consultarAmbiente(idAmbiente);
	}
	
	//Em rel aos assentos de um determinado ambiente
	
	private void inserirLinha(int idAmbiente) throws AmbienteNaoCadastradoException, BancoDeDadosException, ControleExcepttion {
		this.repositorioAmbiente.inserirLinha(idAmbiente);		
	}
	private void removerLinha(int idAmbiente) throws AmbienteNaoCadastradoException, BancoDeDadosException, ControleExcepttion {
		this.repositorioAmbiente.removerLinha(idAmbiente);			
	}	
	private void inserirColuna(int idAmbiente) throws AmbienteNaoCadastradoException, BancoDeDadosException, ControleExcepttion {
		this.repositorioAmbiente.inserirColuna(idAmbiente);	
	}
	private void removerColuna(int idAmbiente) throws AmbienteNaoCadastradoException, BancoDeDadosException, ControleExcepttion {
		this.repositorioAmbiente.removerColuna(idAmbiente);		
	}
	
	public void inserirLinhas(int idAmbiente, int numeroLinhas) throws AmbienteNaoCadastradoException, BancoDeDadosException, ControleExcepttion{
		if(numeroLinhas != 0){
			Ambiente ambiente = this.consultarAmbiente(idAmbiente);			
			if(numeroLinhas >= 0){
				if(ambiente.getLinhas() < numMaxLinhas){
					if((ambiente.getLinhas() + numeroLinhas) <= numMaxLinhas){
						for(int i = 0; i < numeroLinhas; i++){
							this.inserirLinha(idAmbiente);
						}
					}else{
						throw new ControleExcepttion("A sala suporta no mximo "+numMaxLinhas+" linhas. Escolha um valor menor.");
					}
				}else{
					throw new ControleExcepttion("A capacidade mxima de linhas j foi atingida!");
				}
			}else{
				throw new ControleExcepttion("O nmero de linhas a serem inseridas no pode ser negativo");
			}
		}
	}
	
	
	public void removerLinhas(int idAmbiente, int numeroLinhas) throws AmbienteNaoCadastradoException, BancoDeDadosException, ControleExcepttion{
		if(numeroLinhas != 0){
			Ambiente ambiente = this.consultarAmbiente(idAmbiente);
			if(numeroLinhas >= 0){
				if(ambiente.getLinhas() > 1){
					if((ambiente.getLinhas() - numeroLinhas) >= 1){
						for(int i = 0; i < numeroLinhas; i++){
							this.removerLinha(idAmbiente);
						}
					}else{
						throw new ControleExcepttion("A sala suporta no mnimo 1 linha. Escolha um valor menor.");
					}
				}else{
					throw new ControleExcepttion("No existem mais linhas para serem removidas!");
				}
			}else{
				throw new ControleExcepttion("O nmero de linhas a serem removidas no pode ser negativo");
			}
		}
	}

	
	
	public void inserirColunas(int idAmbiente, int numeroColunas) throws AmbienteNaoCadastradoException, BancoDeDadosException, ControleExcepttion{
		if(numeroColunas != 0){
			Ambiente ambiente = this.consultarAmbiente(idAmbiente);
			if(numeroColunas >= 0){
				if(ambiente.getColunas() < numMaxColunas){
					if((ambiente.getColunas() + numeroColunas) <= numMaxColunas){
						for(int i = 0; i < numeroColunas; i++){
							this.inserirColuna(idAmbiente);
						}
					}else{
						throw new ControleExcepttion("A sala suporta no mximo "+numMaxColunas +" colunas. Escolha um valor menor.");
					}
				}else{
					throw new ControleExcepttion("A capacidade mxima de colunas j foi atingida!");
				}
			}else{
				throw new ControleExcepttion("O nmero de colunas a serem inseridas no pode ser negativo");
			}
		}
	}
	
	
	public void removerColunas(int idAmbiente, int numeroColunas) throws AmbienteNaoCadastradoException, BancoDeDadosException, ControleExcepttion{
		if(numeroColunas != 0){
			Ambiente ambiente = this.consultarAmbiente(idAmbiente);
			if(numeroColunas >= 0){
				if(ambiente.getColunas() > 1){
					if((ambiente.getColunas() - numeroColunas) >= 1){
						for(int i = 0; i < numeroColunas; i++){
							this.removerColuna(idAmbiente);
						}
					}else{
						throw new ControleExcepttion("A sala suporta no mnimo 1 coluna. Escolha um valor menor.");
					}
				}else{
					throw new ControleExcepttion("No existem mais colunas para serem removidas!");
				}
			}else{
				throw new ControleExcepttion("O nmero de colunas a serem removidas no pode ser negativo");
			}
		}
	}
	
	public Vector<Ambiente> listarAmbientes() throws BancoDeDadosException{
		return this.repositorioAmbiente.listarAmbientes();
	}
	
}