package negocios;

import java.sql.Connection;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Vector;

import dados.base.*;
import dados.repositorio.RepositorioEvento;
import exceptions.AmbienteNaoCadastradoException;
import exceptions.BancoDeDadosException;
import exceptions.ControleExcepttion;
import exceptions.EventoJahCadastradoException;
import exceptions.EventoNaoCadastradoException;

public class ControleEvento {
	private RepositorioEvento repositorioEvento;
	
	public ControleEvento(Connection conexaoBD, ControleAmbiente controleAmbiente) {
		this.repositorioEvento = new RepositorioEvento(conexaoBD, controleAmbiente);
	}
	
	public void inserirEvento(Evento evento) throws EventoJahCadastradoException, AmbienteNaoCadastradoException, BancoDeDadosException, ControleExcepttion {
		if(!this.dataPassada(evento.getData().getDia() + "/" + evento.getData().getMes() + "/" + evento.getData().getAno() + " " +  evento.getHora().getHoras() + ":" + evento.getHora().getMinutos())){
			if(evento.getId() < 100){
				this.repositorioEvento.inserirEvento(evento);
			}else{
				throw new ControleExcepttion("Utilize um valor menor para o idEvento!");
			}
		}else{
			throw new ControleExcepttion("O evento no pode ser inserido em data e hora passados!");
		}
	}
	
	public void removerEvento(int idEvento) throws EventoNaoCadastradoException, BancoDeDadosException {
		this.repositorioEvento.removerEvento(idEvento);
	}
	
	
	public Evento consultarEvento(int idEvento) throws EventoNaoCadastradoException, AmbienteNaoCadastradoException, BancoDeDadosException{
		return this.repositorioEvento.consultarEvento(idEvento);
	}
	
	public Vector<Evento> consultarEventosPeloNome(String nomeEvento) throws BancoDeDadosException{
		return this.repositorioEvento.consultarEventosPeloNome(nomeEvento);
	}
	
	public Vector<Evento> listarEventos() throws BancoDeDadosException{
		return this.repositorioEvento.listarEventos();
	}
	
	private boolean dataPassada(String data) throws ControleExcepttion {
		   SimpleDateFormat format = new SimpleDateFormat("dd/MM/yyyy");  
		   Date minhaData;
		   Boolean retorno = false;
			try {
				minhaData = format.parse(data);
			  if (minhaData.before(new Date())) {  
			    // O dia j aconteceu  
				   retorno = true;
			   }
			} catch (ParseException e) {
				throw new ControleExcepttion("Data em formato invlido");
			}
			return retorno;  
		}

	public void inserirReserva(int idEvento, int linha, int coluna) throws BancoDeDadosException{{
		this.repositorioEvento.inserirReserva(idEvento, linha, coluna);
	}
	}
}
